@@ -9,7 +9,7 @@ BLOCK_SIZE = 64
 
 def encrypt(mode, key, iv, plaintext):
     encryptor = botan.Cipher(
-        "IDEA/{0}/NoPadding".format(mode), "encrypt", binascii.unhexlify(key)
+        f"IDEA/{mode}/NoPadding", "encrypt", binascii.unhexlify(key)
     )
 
     cipher_text = encryptor.cipher(
@@ -19,7 +19,7 @@ def encrypt(mode, key, iv, plaintext):
 
 
 def verify_vectors(mode, filename):
-    with open(filename, "r") as f:
+    with open(filename) as f:
         vector_file = f.read().splitlines()
 
     vectors = load_nist_vectors(vector_file)
