@@ -7,7 +7,6 @@ from ecdsa import SECP256k1, SigningKey
 from ecdsa.util import sigdecode_der, sigencode_der
 
 from cryptography_vectors import open_vector_file
-
 from tests.utils import load_fips_ecdsa_signing_vectors, load_vectors_from_file
 
 HASHLIB_HASH_TYPES = {
@@ -41,7 +40,7 @@ def build_vectors(fips_vectors):
             continue
 
         yield ""
-        yield "[K-256,{0}]".format(digest_algorithm)
+        yield f"[K-256,{digest_algorithm}]"
         yield ""
 
         for message in messages:
@@ -57,12 +56,12 @@ def build_vectors(fips_vectors):
 
             r, s = sigdecode_der(signature, None)
 
-            yield "Msg = {0}".format(hexlify(message))
-            yield "d = {0:x}".format(secret_key.privkey.secret_multiplier)
-            yield "Qx = {0:x}".format(public_key.pubkey.point.x())
-            yield "Qy = {0:x}".format(public_key.pubkey.point.y())
-            yield "R = {0:x}".format(r)
-            yield "S = {0:x}".format(s)
+            yield f"Msg = {hexlify(message)}"
+            yield f"d = {secret_key.privkey.secret_multiplier:x}"
+            yield f"Qx = {public_key.pubkey.point.x():x}"
+            yield f"Qy = {public_key.pubkey.point.y():x}"
+            yield f"R = {r:x}"
+            yield f"S = {s:x}"
             yield ""
 
 
