@@ -21,27 +21,20 @@ ffi = build_ffi_for_binding(
     modules=[
         # This goes first so we can define some cryptography-wide symbols.
         "cryptography",
-        # Provider comes early as well so we define OSSL_LIB_CTX
-        "provider",
         "asn1",
         "bignum",
         "bio",
-        "cmac",
         "crypto",
         "dh",
         "dsa",
         "ec",
-        "ecdsa",
         "engine",
         "err",
         "evp",
-        "evp_aead",
-        "fips",
         "nid",
         "objects",
         "opensslv",
         "pem",
-        "pkcs12",
         "rand",
         "rsa",
         "ssl",
@@ -49,8 +42,6 @@ ffi = build_ffi_for_binding(
         "x509name",
         "x509v3",
         "x509_vfy",
-        "pkcs7",
-        "callbacks",
     ],
 )
 
