@@ -12,7 +12,9 @@ use cryptography_x509::{
     extensions::{Extension, Extensions},
 };
 
-use crate::{ops::CryptoOps, policy::Policy, ValidationError};
+use crate::{
+    ops::CryptoOps, policy::Policy, ValidationError, ValidationErrorKind, ValidationResult,
+};
 
 pub(crate) struct ExtensionPolicy<B: CryptoOps> {
     pub(crate) authority_information_access: ExtensionValidator<B>,
@@ -26,12 +28,12 @@ pub(crate) struct ExtensionPolicy<B: CryptoOps> {
 }
 
 impl<B: CryptoOps> ExtensionPolicy<B> {
-    pub(crate) fn permits(
+    pub(crate) fn permits<'chain>(
         &self,
         policy: &Policy<'_, B>,
-        cert: &Certificate<'_>,
+        cert: &Certificate<'chain>,
         extensions: &Extensions<'_>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         let mut authority_information_access_seen = false;
         let mut authority_key_identifier_seen = false;
         let mut subject_key_identifier_seen = false;
@@ -81,10 +83,10 @@ impl<B: CryptoOps> ExtensionPolicy<B> {
                     self.extended_key_usage.permits(policy, cert, Some(&ext))?;
                 }
                 _ if ext.critical => {
-                    return Err(ValidationError::ExtensionError {
+                    return Err(ValidationError::new(ValidationErrorKind::ExtensionError {
                         oid: ext.extn_id,
                         reason: "certificate contains unaccounted-for critical extensions",
-                    });
+                    }));
                 }
                 _ => {}
             }
@@ -144,11 +146,17 @@ impl Criticality {
     }
 }
 
-type PresentExtensionValidatorCallback<B> =
-    fn(&Policy<'_, B>, &Certificate<'_>, &Extension<'_>) -> Result<(), ValidationError>;
+type PresentExtensionValidatorCallback<B> = for<'chain> fn(
+    &Policy<'_, B>,
+    &Certificate<'chain>,
+    &Extension<'_>,
+) -> ValidationResult<'chain, (), B>;
 
-type MaybeExtensionValidatorCallback<B> =
-    fn(&Policy<'_, B>, &Certificate<'_>, Option<&Extension<'_>>) -> Result<(), ValidationError>;
+type MaybeExtensionValidatorCallback<B> = for<'chain> fn(
+    &Policy<'_, B>,
+    &Certificate<'chain>,
+    Option<&Extension<'_>>,
+) -> ValidationResult<'chain, (), B>;
 
 /// Represents different validation states for an extension.
 pub(crate) enum ExtensionValidator<B: CryptoOps> {
@@ -195,23 +203,25 @@ impl<B: CryptoOps> ExtensionValidator<B> {
         }
     }
 
-    pub(crate) fn permits(
+    pub(crate) fn permits<'chain>(
         &self,
         policy: &Policy<'_, B>,
-        cert: &Certificate<'_>,
+        cert: &Certificate<'chain>,
         extension: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         match (self, extension) {
             // Extension MUST NOT be present and isn't; OK.
             (ExtensionValidator::NotPresent, None) => Ok(()),
             // Extension MUST NOT be present but is; NOT OK.
-            (ExtensionValidator::NotPresent, Some(extn)) => Err(ValidationError::ExtensionError {
-                oid: extn.extn_id.clone(),
-                reason: "Certificate contains prohibited extension",
-            }),
+            (ExtensionValidator::NotPresent, Some(extn)) => {
+                Err(ValidationError::new(ValidationErrorKind::ExtensionError {
+                    oid: extn.extn_id.clone(),
+                    reason: "Certificate contains prohibited extension",
+                }))
+            }
             // Extension MUST be present but is not; NOT OK.
-            (ExtensionValidator::Present { .. }, None) => Err(ValidationError::Other(
-                "Certificate is missing required extension".to_string(),
+            (ExtensionValidator::Present { .. }, None) => Err(ValidationError::new(
+                ValidationErrorKind::Other("Certificate is missing required extension".to_string()),
             )),
             // Extension MUST be present and is; check it.
             (
@@ -222,10 +232,10 @@ impl<B: CryptoOps> ExtensionValidator<B> {
                 Some(extn),
             ) => {
                 if !criticality.permits(extn.critical) {
-                    return Err(ValidationError::ExtensionError {
+                    return Err(ValidationError::new(ValidationErrorKind::ExtensionError {
                         oid: extn.extn_id.clone(),
                         reason: "Certificate extension has incorrect criticality",
-                    });
+                    }));
                 }
 
                 // If a custom validator is supplied, apply it.
@@ -242,10 +252,10 @@ impl<B: CryptoOps> ExtensionValidator<B> {
                 match extn {
                     // If the extension is present, apply our criticality check.
                     Some(extn) if !criticality.permits(extn.critical) => {
-                        Err(ValidationError::ExtensionError {
+                        Err(ValidationError::new(ValidationErrorKind::ExtensionError {
                             oid: extn.extn_id.clone(),
                             reason: "Certificate extension has incorrect criticality",
-                        })
+                        }))
                     }
                     // If a custom validator is supplied, apply it.
                     _ => validator.map_or(Ok(()), |v| v(policy, cert, extn)),
@@ -265,44 +275,44 @@ pub(crate) mod ee {
 
     use crate::{
         ops::CryptoOps,
-        policy::{Policy, ValidationError},
+        policy::{Policy, ValidationError, ValidationErrorKind, ValidationResult},
     };
 
-    pub(crate) fn basic_constraints<B: CryptoOps>(
+    pub(crate) fn basic_constraints<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
             let basic_constraints: BasicConstraints = extn.value()?;
 
             if basic_constraints.ca {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "basicConstraints.cA must not be asserted in an EE certificate".to_string(),
-                ));
+                )));
             }
         }
 
         Ok(())
     }
 
-    pub(crate) fn subject_alternative_name<B: CryptoOps>(
+    pub(crate) fn subject_alternative_name<'chain, B: CryptoOps>(
         policy: &Policy<'_, B>,
         cert: &Certificate<'_>,
         extn: &Extension<'_>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         match (cert.subject().is_empty(), extn.critical) {
             // If the subject is empty, the SAN MUST be critical.
             (true, false) => {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "EE subjectAltName MUST be critical when subject is empty".to_string(),
-                ));
+                )));
             }
             // If the subject is non-empty, the SAN MUST NOT be critical.
             (false, true) => {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "EE subjectAltName MUST NOT be critical when subject is nonempty".to_string(),
-                ))
+                )))
             }
             _ => (),
         };
@@ -314,20 +324,20 @@ pub(crate) mod ee {
         if let Some(sub) = policy.subject.as_ref() {
             let san: SubjectAlternativeName<'_> = extn.value()?;
             if !sub.matches(&san) {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "leaf certificate has no matching subjectAltName".into(),
-                ));
+                )));
             }
         }
 
         Ok(())
     }
 
-    pub(crate) fn extended_key_usage<B: CryptoOps>(
+    pub(crate) fn extended_key_usage<'chain, B: CryptoOps>(
         policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
             let mut ekus: ExtendedKeyUsage<'_> = extn.value()?;
 
@@ -340,25 +350,27 @@ pub(crate) mod ee {
             if ekus.any(|eku| eku == policy.extended_key_usage) {
                 Ok(())
             } else {
-                Err(ValidationError::Other("required EKU not found".to_string()))
+                Err(ValidationError::new(ValidationErrorKind::Other(
+                    "required EKU not found".to_string(),
+                )))
             }
         } else {
             Ok(())
         }
     }
 
-    pub(crate) fn key_usage<B: CryptoOps>(
+    pub(crate) fn key_usage<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
             let key_usage: KeyUsage<'_> = extn.value()?;
 
             if key_usage.key_cert_sign() {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "EE keyUsage must not assert keyCertSign".to_string(),
-                ));
+                )));
             }
         }
 
@@ -369,6 +381,7 @@ pub(crate) mod ee {
 pub(crate) mod ca {
     use cryptography_x509::{
         certificate::Certificate,
+        common::Asn1Read,
         extensions::{
             AuthorityKeyIdentifier, BasicConstraints, ExtendedKeyUsage, Extension, KeyUsage,
             NameConstraints,
@@ -378,14 +391,14 @@ pub(crate) mod ca {
 
     use crate::{
         ops::CryptoOps,
-        policy::{Policy, ValidationError},
+        policy::{Policy, ValidationError, ValidationErrorKind, ValidationResult},
     };
 
-    pub(crate) fn authority_key_identifier<B: CryptoOps>(
+    pub(crate) fn authority_key_identifier<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         // CABF: AKI is required on all CA certificates *except* root CA certificates,
         // where is it merely recommended. This is slightly different from RFC 5280,
         // which requires AKI on all CA certificates *except* self-signed root CA certificates.
@@ -401,61 +414,56 @@ pub(crate) mod ca {
         // some chains that are not strictly CABF compliant (e.g. ones where intermediate
         // CAs are missing AKIs), but this is a relatively minor discrepancy.
         if let Some(extn) = extn {
-            let aki: AuthorityKeyIdentifier<'_> = extn.value()?;
+            let aki: AuthorityKeyIdentifier<'_, Asn1Read> = extn.value()?;
             // 7.1.2.11.1 Authority Key Identifier:
 
             // keyIdentifier MUST be present.
             // TODO: Check that keyIdentifier matches subjectKeyIdentifier.
             if aki.key_identifier.is_none() {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "authorityKeyIdentifier must contain keyIdentifier".to_string(),
-                ));
+                )));
             }
 
-            // authorityCertIssuer and authorityCertSerialNumber MUST NOT be present.
-            if aki.authority_cert_issuer.is_some() {
-                return Err(ValidationError::Other(
-                    "authorityKeyIdentifier must not contain authorityCertIssuer".to_string(),
-                ));
-            }
-
-            if aki.authority_cert_serial_number.is_some() {
-                return Err(ValidationError::Other(
-                    "authorityKeyIdentifier must not contain authorityCertSerialNumber".to_string(),
-                ));
-            }
+            // NOTE: CABF 7.1.2.1.3 says that Root CAs MUST NOT
+            // have authorityCertIdentifier or authorityCertSerialNumber,
+            // but these are present in practice in trust program bundles
+            // due to older roots that have been grandfathered in.
+            // Other validators are permissive of these being present,
+            // so we don't check for them.
+            // See #11461 for more information.
         }
 
         Ok(())
     }
 
-    pub(crate) fn key_usage<B: CryptoOps>(
+    pub(crate) fn key_usage<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: &Extension<'_>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         let key_usage: KeyUsage<'_> = extn.value()?;
 
         if !key_usage.key_cert_sign() {
-            return Err(ValidationError::Other(
+            return Err(ValidationError::new(ValidationErrorKind::Other(
                 "keyUsage.keyCertSign must be asserted in a CA certificate".to_string(),
-            ));
+            )));
         }
 
         Ok(())
     }
 
-    pub(crate) fn basic_constraints<B: CryptoOps>(
+    pub(crate) fn basic_constraints<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: &Extension<'_>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         let basic_constraints: BasicConstraints = extn.value()?;
 
         if !basic_constraints.ca {
-            return Err(ValidationError::Other(
+            return Err(ValidationError::new(ValidationErrorKind::Other(
                 "basicConstraints.cA must be asserted in a CA certificate".to_string(),
-            ));
+            )));
         }
 
         // NOTE: basicConstraints.pathLength is checked as part of
@@ -465,28 +473,28 @@ pub(crate) mod ca {
         Ok(())
     }
 
-    pub(crate) fn name_constraints<B: CryptoOps>(
+    pub(crate) fn name_constraints<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
-            let name_constraints: NameConstraints<'_> = extn.value()?;
+            let name_constraints: NameConstraints<'_, Asn1Read> = extn.value()?;
 
             let permitted_subtrees_empty = name_constraints
                 .permitted_subtrees
                 .as_ref()
-                .map_or(true, |pst| pst.unwrap_read().is_empty());
+                .map_or(true, |pst| pst.is_empty());
             let excluded_subtrees_empty = name_constraints
                 .excluded_subtrees
                 .as_ref()
-                .map_or(true, |est| est.unwrap_read().is_empty());
+                .map_or(true, |est| est.is_empty());
 
             if permitted_subtrees_empty && excluded_subtrees_empty {
-                return Err(ValidationError::Other(
+                return Err(ValidationError::new(ValidationErrorKind::Other(
                     "nameConstraints must have non-empty permittedSubtrees or excludedSubtrees"
                         .to_string(),
-                ));
+                )));
             }
 
             // NOTE: Both RFC 5280 and CABF require each `GeneralSubtree`
@@ -497,11 +505,11 @@ pub(crate) mod ca {
         Ok(())
     }
 
-    pub(crate) fn extended_key_usage<B: CryptoOps>(
+    pub(crate) fn extended_key_usage<'chain, B: CryptoOps>(
         policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
             let mut ekus: ExtendedKeyUsage<'_> = extn.value()?;
 
@@ -510,7 +518,9 @@ pub(crate) mod ca {
             if ekus.any(|eku| eku == policy.extended_key_usage || eku == EKU_ANY_KEY_USAGE_OID) {
                 Ok(())
             } else {
-                Err(ValidationError::Other("required EKU not found".to_string()))
+                Err(ValidationError::new(ValidationErrorKind::Other(
+                    "required EKU not found".to_string(),
+                )))
             }
         } else {
             Ok(())
@@ -521,23 +531,24 @@ pub(crate) mod ca {
 pub(crate) mod common {
     use cryptography_x509::{
         certificate::Certificate,
+        common::Asn1Read,
         extensions::{Extension, SequenceOfAccessDescriptions},
     };
 
     use crate::{
         ops::CryptoOps,
-        policy::{Policy, ValidationError},
+        policy::{Policy, ValidationResult},
     };
 
-    pub(crate) fn authority_information_access<B: CryptoOps>(
+    pub(crate) fn authority_information_access<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         extn: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         if let Some(extn) = extn {
             // We don't currently do anything useful with these, but we
             // do check that they're well-formed.
-            let _: SequenceOfAccessDescriptions<'_> = extn.value()?;
+            let _: SequenceOfAccessDescriptions<'_, Asn1Read> = extn.value()?;
         }
 
         Ok(())
@@ -555,7 +566,7 @@ mod tests {
     use crate::certificate::tests::PublicKeyErrorOps;
     use crate::ops::tests::{cert, v1_cert_pem};
     use crate::ops::CryptoOps;
-    use crate::policy::{Policy, Subject, ValidationError};
+    use crate::policy::{Policy, Subject, ValidationResult};
     use crate::types::DNSName;
 
     #[test]
@@ -582,7 +593,7 @@ mod tests {
         critical: bool,
         ext: &T,
     ) -> Vec<u8> {
-        let ext_value = asn1::write_single(&ext).unwrap();
+        let ext_value = asn1::write_single(ext).unwrap();
         let ext = Extension {
             extn_id: oid,
             critical,
@@ -591,11 +602,11 @@ mod tests {
         asn1::write_single(&ext).unwrap()
     }
 
-    fn present_extension_validator<B: CryptoOps>(
+    fn present_extension_validator<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         _ext: &Extension<'_>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         Ok(())
     }
 
@@ -631,11 +642,11 @@ mod tests {
         assert!(extension_validator.permits(&policy, &cert, None).is_err());
     }
 
-    fn maybe_extension_validator<B: CryptoOps>(
+    fn maybe_extension_validator<'chain, B: CryptoOps>(
         _policy: &Policy<'_, B>,
         _cert: &Certificate<'_>,
         _ext: Option<&Extension<'_>>,
-    ) -> Result<(), ValidationError> {
+    ) -> ValidationResult<'chain, (), B> {
         Ok(())
     }
 
