@@ -69,17 +69,13 @@ impl X448PrivateKey {
         let mut deriver = openssl::derive::Deriver::new(&self.pkey)?;
         deriver.set_peer(&peer_public_key.pkey)?;
 
-        Ok(pyo3::types::PyBytes::new_bound_with(
-            py,
-            deriver.len()?,
-            |b| {
-                let n = deriver.derive(b).map_err(|_| {
-                    pyo3::exceptions::PyValueError::new_err("Error computing shared key.")
-                })?;
-                assert_eq!(n, b.len());
-                Ok(())
-            },
-        )?)
+        Ok(pyo3::types::PyBytes::new_with(py, deriver.len()?, |b| {
+            let n = deriver.derive(b).map_err(|_| {
+                pyo3::exceptions::PyValueError::new_err("Error computing shared key.")
+            })?;
+            assert_eq!(n, b.len());
+            Ok(())
+        })?)
     }
 
     fn public_key(&self) -> CryptographyResult<X448PublicKey> {
@@ -97,7 +93,7 @@ impl X448PrivateKey {
         py: pyo3::Python<'p>,
     ) -> CryptographyResult<pyo3::Bound<'p, pyo3::types::PyBytes>> {
         let raw_bytes = self.pkey.raw_private_key()?;
-        Ok(pyo3::types::PyBytes::new_bound(py, &raw_bytes))
+        Ok(pyo3::types::PyBytes::new(py, &raw_bytes))
     }
 
     fn private_bytes<'p>(
@@ -127,7 +123,7 @@ impl X448PublicKey {
         py: pyo3::Python<'p>,
     ) -> CryptographyResult<pyo3::Bound<'p, pyo3::types::PyBytes>> {
         let raw_bytes = self.pkey.raw_public_key()?;
-        Ok(pyo3::types::PyBytes::new_bound(py, &raw_bytes))
+        Ok(pyo3::types::PyBytes::new(py, &raw_bytes))
     }
 
     fn public_bytes<'p>(
