@@ -7,19 +7,17 @@ import binascii
 
 import pytest
 
-from cryptography.hazmat.backends.interfaces import HMACBackend
 from cryptography.hazmat.primitives import hashes, hmac
 
-from .utils import generate_hmac_test
 from ...utils import load_hash_vectors
+from .utils import generate_hmac_test
 
 
 @pytest.mark.supported(
     only_if=lambda backend: backend.hmac_supported(hashes.MD5()),
     skip_message="Does not support MD5",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACMD5(object):
+class TestHMACMD5:
     test_hmac_md5 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -32,8 +30,7 @@ class TestHMACMD5(object):
     only_if=lambda backend: backend.hmac_supported(hashes.SHA1()),
     skip_message="Does not support SHA1",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACSHA1(object):
+class TestHMACSHA1:
     test_hmac_sha1 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -46,8 +43,7 @@ class TestHMACSHA1(object):
     only_if=lambda backend: backend.hmac_supported(hashes.SHA224()),
     skip_message="Does not support SHA224",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACSHA224(object):
+class TestHMACSHA224:
     test_hmac_sha224 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -60,8 +56,7 @@ class TestHMACSHA224(object):
     only_if=lambda backend: backend.hmac_supported(hashes.SHA256()),
     skip_message="Does not support SHA256",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACSHA256(object):
+class TestHMACSHA256:
     test_hmac_sha256 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -74,8 +69,7 @@ class TestHMACSHA256(object):
     only_if=lambda backend: backend.hmac_supported(hashes.SHA384()),
     skip_message="Does not support SHA384",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACSHA384(object):
+class TestHMACSHA384:
     test_hmac_sha384 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -88,8 +82,7 @@ class TestHMACSHA384(object):
     only_if=lambda backend: backend.hmac_supported(hashes.SHA512()),
     skip_message="Does not support SHA512",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACSHA512(object):
+class TestHMACSHA512:
     test_hmac_sha512 = generate_hmac_test(
         load_hash_vectors,
         "HMAC",
@@ -104,8 +97,7 @@ class TestHMACSHA512(object):
     ),
     skip_message="Does not support BLAKE2",
 )
-@pytest.mark.requires_backend_interface(interface=HMACBackend)
-class TestHMACBLAKE2(object):
+class TestHMACBLAKE2:
     def test_blake2b(self, backend):
         h = hmac.HMAC(b"0" * 64, hashes.BLAKE2b(digest_size=64), backend)
         h.update(b"test")
