@@ -19,14 +19,14 @@ from ...utils import load_vectors_from_file, load_x963_vectors
 def _skip_hashfn_unsupported(backend, hashfn):
     if not backend.hash_supported(hashfn):
         pytest.skip(
-            "Hash {} is not supported by this backend {}".format(
-                hashfn.name, backend
-            )
+            f"Hash {hashfn.name} is not supported by this backend {backend}"
         )
 
 
 class TestX963:
-    _algorithms_dict: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
+    _algorithms_dict: typing.ClassVar[
+        typing.Dict[str, typing.Type[hashes.HashAlgorithm]]
+    ] = {
         "SHA-1": hashes.SHA1,
         "SHA-224": hashes.SHA224,
         "SHA-256": hashes.SHA256,
