@@ -2,10 +2,10 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import sys
 import types
+import typing
 import warnings
 
 import pytest
@@ -13,11 +13,13 @@ import pytest
 from cryptography.utils import deprecated
 
 
-class TestDeprecated(object):
+class TestDeprecated:
+    @typing.no_type_check
     def test_deprecated(self, monkeypatch):
         mod = types.ModuleType("TestDeprecated/test_deprecated")
         monkeypatch.setitem(sys.modules, mod.__name__, mod)
-        mod.X = deprecated(
+        deprecated(
+            name="X",
             value=1,
             module_name=mod.__name__,
             message="deprecated message text",
@@ -48,10 +50,12 @@ class TestDeprecated(object):
 
         assert "Y" in dir(mod)
 
+    @typing.no_type_check
     def test_deleting_deprecated_members(self, monkeypatch):
         mod = types.ModuleType("TestDeprecated/test_deprecated")
         monkeypatch.setitem(sys.modules, mod.__name__, mod)
-        mod.X = deprecated(
+        deprecated(
+            name="X",
             value=1,
             module_name=mod.__name__,
             message="deprecated message text",
